from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.utils.deprecation import MiddlewareMixin

from ridi_django_oauth2.utils.token import get_token_from_cookie, get_token_info


class AuthenticationMiddleware(MiddlewareMixin):
    def process_request(self, request):
        request.user = AnonymousUser()

        token = get_token_from_cookie(request=request)
        token_info = None
        if token.access_token:
            token_info = get_token_info(token.access_token.token)

        if token_info is not None:
            user, _ = get_user_model().objects.get_or_create(u_idx=token_info.u_idx)
            user.token = token
            user.token_info = token_info
            request.user = user

        return None



import typing

from ridi_django_oauth2.config import RidiOAuth2Config


def generate_cookie(access_token: str) -> typing.Dict:
    return {
        RidiOAuth2Config.get_access_token_cookie_key(): access_token
    }



import typing

from django.http import HttpRequest

from ridi_django_oauth2.config import RidiOAuth2Config
from ridi_oauth2.client.dtos import TokenData
from ridi_oauth2.introspector.dtos import AccessTokenInfo
from ridi_oauth2.introspector.exceptions import ExpireTokenException, InvalidJwtSignatureException
from ridi_oauth2.introspector.helpers import JwtIntrospectHelper


def get_token_from_cookie(request: HttpRequest) -> TokenData:
    return TokenData.from_dict({
        'access_token': _get_access_token_from_cookie(request=request),
        'refresh_token': _get_refresh_token_from_cookie(request=request),
    })


def get_token_info(token: str) -> typing.Optional[AccessTokenInfo]:
    try:
        token_info = JwtIntrospectHelper.introspect(jwt_info=RidiOAuth2Config.get_jwt_info(), access_token=token)
    except (KeyError, ExpireTokenException, InvalidJwtSignatureException):
        token_info = None

    return token_info


def _get_access_token_from_cookie(request: HttpRequest) -> str:
    return request.COOKIES.get(RidiOAuth2Config.get_access_token_cookie_key())


def _get_refresh_token_from_cookie(request: HttpRequest) -> str:
    return request.COOKIES.get(RidiOAuth2Config.get_refresh_token_cookie_key())



import typing
from datetime import datetime


class JwtInfo:
    def __init__(self, secret: str, algorithm: str):
        self._secret = secret
        self._algorithm = algorithm

    @property
    def secret(self) -> str:
        return self._secret

    @property
    def algorithm(self) -> str:
        return self._algorithm


class AccessTokenInfo:
    def __init__(self, subject: str, u_idx: int, expire: int, client_id: str, scope: typing.List):
        self._subject = subject
        self._u_idx = u_idx
        self._expire_timestamp = expire
        self._expire_date = datetime.fromtimestamp(expire)
        self._client_id = client_id
        self._scope = scope

    @property
    def subject(self) -> str:
        return self._subject

    @property
    def u_idx(self) -> int:
        return self._u_idx

    @property
    def expire_timestamp(self) -> int:
        return self._expire_timestamp

    @property
    def expire_date(self) -> datetime:
        return self._expire_date

    @property
    def client_id(self) -> str:
        return self._client_id

    @property
    def scope(self) -> typing.List:
        return self._scope

    @staticmethod
    def from_dict(dictionary: typing.Dict):
        return AccessTokenInfo(
            subject=dictionary['sub'], u_idx=dictionary['u_idx'], expire=dictionary['exp'], client_id=dictionary['client_id'],
            scope=dictionary['scope'],
        )



from rest_framework.authentication import BaseAuthentication


class OAuth2Authentication(BaseAuthentication):

    def authenticate(self, request):
        user = getattr(request._request, 'user', None)

        if not user or not user.is_active:
            return None

        return user, None




import typing

import jwt

from ridi_oauth2.common.constants import TokenType
from ridi_oauth2.introspector.dtos import JwtInfo
from ridi_oauth2.introspector.exceptions import ExpireTokenException, InvalidJwtSignatureException
from .base import BaseIntrospector


class JwtIntrospector(BaseIntrospector):
    _DEFAULT_SCOPE_DELIMITER = ' '

    def __init__(self, jwt_info: JwtInfo, access_token: str):
        self._jwt_info = jwt_info
        super().__init__(access_token=access_token, token_type_hint=TokenType.BEARER)

    def introspect(self) -> typing.Dict:
        try:
            payload = jwt.decode(jwt=self.access_token, key=self._jwt_info.secret, algorithms=[self._jwt_info.algorithm])
        except jwt.exceptions.ExpiredSignatureError:
            raise ExpireTokenException
        except jwt.exceptions.DecodeError:
            raise InvalidJwtSignatureException
        except jwt.InvalidAlgorithmError:
            raise InvalidJwtSignatureException

        payload = self._active_response(payload=payload)
        payload = self._split_scopes(payload=payload)
        return payload

    @staticmethod
    def _active_response(payload: typing.Dict) -> typing.Dict:
        payload.update({'active': True})
        return payload

    @classmethod
    def _split_scopes(cls, payload: typing.Dict) -> typing.Dict:
        if payload.get('scope', None) is None:
            return payload

        if isinstance(payload['scope'], list):
            return payload

        payload['scope'] = payload['scope'].split(cls._DEFAULT_SCOPE_DELIMITER)
        return payload



from django.conf import settings

from ridi_oauth2.introspector.dtos import JwtInfo


class _Settings:
    JWT_SECRET_NAME = 'RIDI_OAUTH2_JWT_SECRET'
    JWT_ALGORITHM_NAME = 'RIDI_OAUTH2_JWT_ALGORITHM'

    COOKIE_DOMAIN = 'RIDI_OAUTH2_COOKIE_DOMAIN'
    ACCESS_TOKEN_COOKIE_KEY = 'RIDI_OAUTH2_ACCESS_TOKEN_COOKIE_KEY'
    REFRESH_TOKEN_COOKIE_KEY = 'RIDI_OAUTH2_REFRESH_TOKEN_COOKIE_KEY'


class _Default:
    JWT_ALGORITHM = 'HS256'

    COOKIE_DOMAIN = 'ridibooks.com'
    ACCESS_TOKEN_COOKIE_KEY = "ridi-at"
    REFRESH_TOKEN_COOKIE_KEY = "ridi-rt"


# JwtInfo
_RIDI_OAUTH2_JWT_SECRET = getattr(settings, _Settings.JWT_SECRET_NAME)
_RIDI_OAUTH2_JWT_ALGORITHM = getattr(settings, _Settings.JWT_ALGORITHM_NAME, _Default.JWT_ALGORITHM)

_JWT_INFO = JwtInfo(secret=_RIDI_OAUTH2_JWT_SECRET, algorithm=_RIDI_OAUTH2_JWT_ALGORITHM)

# Cookie
_RIDI_COOKIE_DOMAIN = getattr(settings, _Settings.COOKIE_DOMAIN, _Default.COOKIE_DOMAIN)
_RIDI_ACCESS_TOKEN_COOKIE_KEY = getattr(settings, _Settings.ACCESS_TOKEN_COOKIE_KEY, _Default.ACCESS_TOKEN_COOKIE_KEY)
_RIDI_REFRESH_TOKEN_COOKIE_KEY = getattr(settings, _Settings.REFRESH_TOKEN_COOKIE_KEY, _Default.REFRESH_TOKEN_COOKIE_KEY)


class RidiOAuth2Config:
    @staticmethod
    def get_jwt_info() -> JwtInfo:
        return _JWT_INFO

    @staticmethod
    def get_cookie_domain() -> str:
        return _RIDI_COOKIE_DOMAIN

    @staticmethod
    def get_access_token_cookie_key() -> str:
        return _RIDI_ACCESS_TOKEN_COOKIE_KEY

    @staticmethod
    def get_refresh_token_cookie_key() -> str:
        return _RIDI_REFRESH_TOKEN_COOKIE_KEY





class InvalidResponseException(Exception):
    pass


class NotSupportedGrantException(Exception):
    pass


class OAuthFailureException(Exception):
    def __init__(self, error_code: str, *args, description: str=None, error_uri: str=None):
        self._error_code = error_code
        self._description = description
        self._error_uri = error_uri

        super().__init__(*args)

    @property
    def error_code(self) -> str:
        return self._error_code

    @property
    def description(self) -> str:
        return self._description

    @property
    def error_uri(self) -> str:
        return self._error_uri



# Generated by Django 2.0.2 on 2018-03-22 02:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ridi_django_oauth2', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ridiuser',
            name='u_idx',
            field=models.IntegerField(editable=False, primary_key=True, serialize=False, verbose_name='u_idx'),
        ),
    ]



from ridi_oauth2.introspector.dtos import AccessTokenInfo, JwtInfo
from ridi_oauth2.introspector.exceptions import InvalidJwtSignatureException
from ridi_oauth2.introspector.jwt_introspector import JwtIntrospector


class JwtIntrospectHelper:
    @staticmethod
    def introspect(jwt_info: JwtInfo, access_token: str) -> AccessTokenInfo:
        introspector = JwtIntrospector(jwt_info=jwt_info, access_token=access_token)
        result = introspector.introspect()
        try:
            return AccessTokenInfo.from_dict(result)
        except KeyError:
            raise InvalidJwtSignatureException
